==> hackmud-script-manager-0.21.1-698970b.tgz: package/processScript/index.js <==
import { relative, isAbsolute, sep } from "path"
import o8o18ueduqgposmxo6khc98u from "@babel/generator"
import { parse } from "@babel/parser"
import yerg92h3xef0pxntmentis4x from "@babel/plugin-proposal-decorators"
import bte791wmvheh6c8duknw52o8 from "@babel/plugin-proposal-destructuring-private"
import hh8bh9as603wznftfw598ev4 from "@babel/plugin-proposal-explicit-resource-management"
import mw6tmscbfy336xel0wwko3m5 from "@babel/plugin-transform-class-properties"
import hqa52cv6hq1ln2kxplifj0vt from "@babel/plugin-transform-class-static-block"
import kue2jd7y9faemw8fy3f873jc from "@babel/plugin-transform-exponentiation-operator"
import rla2xnm97c6qkc9c3lugzrsn from "@babel/plugin-transform-json-strings"
import yrmvvy1iqsvv49eiqoaiagw1 from "@babel/plugin-transform-logical-assignment-operators"
import u3y6roa2q93oh9vgn7fh5bp8 from "@babel/plugin-transform-nullish-coalescing-operator"
import q5clnfu3mx2a2utmxqlodg7x from "@babel/plugin-transform-numeric-separator"
import w96tm2oib174p8k4grehe3y1 from "@babel/plugin-transform-object-rest-spread"
import zk6pwiooyiv76c30twyn2zhq from "@babel/plugin-transform-optional-catch-binding"
import l8e6321b0ayw5a4k0tzzxigq from "@babel/plugin-transform-optional-chaining"
import nmo6kym6bvsq78mjyglii3dt from "@babel/plugin-transform-private-property-in-object"
import b87sx47xj4o4ode3kr2ona7l from "@babel/plugin-transform-unicode-sets-regex"
import gpiwud5ibbmji5fxkev1yx1i from "@babel/traverse"
import t from "@babel/types"
import rollupPluginAlias from "@rollup/plugin-alias"
import { babel } from "@rollup/plugin-babel"
import rollupPluginCommonJS from "@rollup/plugin-commonjs"
import rollupPluginJSON from "@rollup/plugin-json"
import rollupPluginNodeResolve from "@rollup/plugin-node-resolve"
import { assert } from "@samual/lib/assert"
import prettier from "prettier"
import { rollup } from "rollup"
import { supportedExtensions } from "../constants.js"
import { minify } from "./minify.js"
import { postprocess } from "./postprocess.js"
import { preprocess } from "./preprocess.js"
import { getReferencePathsToGlobal, includesIllegalString, replaceUnsafeStrings } from "./shared.js"
import { transform } from "./transform.js"
import "@samual/lib/countHackmudCharacters"
import "@samual/lib/spliceString"
import "acorn"
import "terser"
import "import-meta-resolve"
import "@samual/lib/clearObject"
const generate = o8o18ueduqgposmxo6khc98u.default,
	babelPluginProposalDecorators = yerg92h3xef0pxntmentis4x.default,
	babelPluginProposalDestructuringPrivate = bte791wmvheh6c8duknw52o8.default,
	babelPluginProposalExplicitResourceManagement = hh8bh9as603wznftfw598ev4.default,
	babelPluginTransformClassProperties = mw6tmscbfy336xel0wwko3m5.default,
	babelPluginTransformClassStaticBlock = hqa52cv6hq1ln2kxplifj0vt.default,
	babelPluginTransformExponentiationOperator = kue2jd7y9faemw8fy3f873jc.default,
	babelPluginTransformJsonStrings = rla2xnm97c6qkc9c3lugzrsn.default,
	babelPluginTransformLogicalAssignmentOperators = yrmvvy1iqsvv49eiqoaiagw1.default,
	babelPluginTransformNullishCoalescingOperator = u3y6roa2q93oh9vgn7fh5bp8.default,
	babelPluginTransformNumericSeparator = q5clnfu3mx2a2utmxqlodg7x.default,
	babelPluginTransformObjectRestSpread = w96tm2oib174p8k4grehe3y1.default,
	babelPluginTransformOptionalCatchBinding = zk6pwiooyiv76c30twyn2zhq.default,
	babelPluginTransformOptionalChaining = l8e6321b0ayw5a4k0tzzxigq.default,
	babelPluginTransformPrivatePropertyInObject = nmo6kym6bvsq78mjyglii3dt.default,
	babelPluginTransformUnicodeSetsRegex = b87sx47xj4o4ode3kr2ona7l.default,
	traverse = gpiwud5ibbmji5fxkev1yx1i.default,
	{ format } = prettier
async function processScript(
	code,
	{
		minify: shouldMinify = !0,
		uniqueId = Math.floor(Math.random() * 2 ** 52)
			.toString(36)
			.padStart(11, "0"),
		scriptUser,
		scriptName,
		filePath,
		mangleNames = !1,
		forceQuineCheats,
		rootFolderPath
	}
) {
	assert(/^\w{11}$/.exec(uniqueId), "src/processScript/index.ts:82:36")
	const sourceCode = code
	let autocomplete, statedSeclevel
	const autocompleteMatch = /^function\s*\(.+\/\/(?<autocomplete>.+)/.exec(code)
	if (autocompleteMatch) {
		code = "export default " + code
		;({ autocomplete } = autocompleteMatch.groups)
	} else
		for (const line of code.split("\n")) {
			const comment = /^\s*\/\/(?<commentContent>.+)/.exec(line)
			if (!comment) break
			const commentContent = comment.groups.commentContent.trim()
			if (commentContent.startsWith("@autocomplete ")) autocomplete = commentContent.slice(14).trimStart()
			else if (commentContent.startsWith("@seclevel ")) {
				const seclevelString = commentContent.slice(10).trimStart().toLowerCase()
				switch (seclevelString) {
					case "fullsec":
					case "full":
					case "fs":
					case "4s":
					case "f":
					case "4":
						statedSeclevel = 4
						break
					case "highsec":
					case "high":
					case "hs":
					case "3s":
					case "h":
					case "3":
						statedSeclevel = 3
						break
					case "midsec":
					case "mid":
					case "ms":
					case "2s":
					case "m":
					case "2":
						statedSeclevel = 2
						break
					case "lowsec":
					case "low":
					case "ls":
					case "1s":
					case "l":
					case "1":
						statedSeclevel = 1
						break
					case "nullsec":
					case "null":
					case "ns":
					case "0s":
					case "n":
					case "0":
						statedSeclevel = 0
						break
					default:
						throw Error(`unrecognised seclevel "${seclevelString}"`)
				}
			}
		}
	assert(/^\w{11}$/.exec(uniqueId), "src/processScript/index.ts:161:36")
	const plugins = [
		[babelPluginProposalDecorators, { decoratorsBeforeExport: !0 }],
		[babelPluginTransformClassProperties],
		[babelPluginTransformClassStaticBlock],
		[babelPluginTransformPrivatePropertyInObject],
		[babelPluginTransformLogicalAssignmentOperators],
		[babelPluginTransformNumericSeparator],
		[babelPluginTransformNullishCoalescingOperator],
		[babelPluginTransformOptionalChaining],
		[babelPluginTransformOptionalCatchBinding],
		[babelPluginTransformJsonStrings],
		[babelPluginTransformObjectRestSpread],
		[babelPluginTransformExponentiationOperator],
		[babelPluginTransformUnicodeSetsRegex],
		[babelPluginProposalDestructuringPrivate],
		[babelPluginProposalExplicitResourceManagement]
	]
	let filePathResolved
	if (filePath) {
		filePathResolved = relative(".", filePath)
		if (filePath.endsWith(".ts"))
			plugins.unshift([
				await import("@babel/plugin-transform-typescript").then(module => module.default),
				{ allowDeclareFields: !0, optimizeConstEnums: !0 }
			])
		else {
			const [
				babelPluginProposalDoExpressions,
				babelPluginProposalFunctionBind,
				babelPluginProposalFunctionSent,
				babelPluginProposalPartialApplication,
				babelPluginProposalPipelineOperator,
				babelPluginProposalThrowExpressions,
				babelPluginProposalRecordAndTuple
			] = await Promise.all([
				import("@babel/plugin-proposal-do-expressions").then(module => module.default),
				import("@babel/plugin-proposal-function-bind").then(module => module.default),
				import("@babel/plugin-proposal-function-sent").then(module => module.default),
				import("@babel/plugin-proposal-partial-application").then(module => module.default),
				import("@babel/plugin-proposal-pipeline-operator").then(module => module.default),
				import("@babel/plugin-proposal-throw-expressions").then(module => module.default),
				import("@babel/plugin-proposal-record-and-tuple").then(module => module.default)
			])
			plugins.push(
				[babelPluginProposalDoExpressions],
				[babelPluginProposalFunctionBind],
				[babelPluginProposalFunctionSent],
				[babelPluginProposalPartialApplication],
				[babelPluginProposalPipelineOperator, { proposal: "hack", topicToken: "%" }],
				[babelPluginProposalThrowExpressions],
				[babelPluginProposalRecordAndTuple, { syntaxType: "hash", importPolyfill: !0 }]
			)
		}
	} else {
		filePathResolved = uniqueId + ".ts"
		const [
			babelPluginTransformTypescript,
			babelPluginProposalDoExpressions,
			babelPluginProposalFunctionBind,
			babelPluginProposalFunctionSent,
			babelPluginProposalPartialApplication,
			babelPluginProposalPipelineOperator,
			babelPluginProposalThrowExpressions,
			babelPluginProposalRecordAndTuple
		] = await Promise.all([
			import("@babel/plugin-transform-typescript").then(module => module.default),
			import("@babel/plugin-proposal-do-expressions").then(module => module.default),
			import("@babel/plugin-proposal-function-bind").then(module => module.default),
			import("@babel/plugin-proposal-function-sent").then(module => module.default),
			import("@babel/plugin-proposal-partial-application").then(module => module.default),
			import("@babel/plugin-proposal-pipeline-operator").then(module => module.default),
			import("@babel/plugin-proposal-throw-expressions").then(module => module.default),
			import("@babel/plugin-proposal-record-and-tuple").then(module => module.default)
		])
		plugins.push(
			[babelPluginTransformTypescript, { allowDeclareFields: !0, optimizeConstEnums: !0 }],
			[babelPluginProposalDoExpressions],
			[babelPluginProposalFunctionBind],
			[babelPluginProposalFunctionSent],
			[babelPluginProposalPartialApplication],
			[babelPluginProposalPipelineOperator, { proposal: "hack", topicToken: "%" }],
			[babelPluginProposalThrowExpressions],
			[babelPluginProposalRecordAndTuple, { syntaxType: "hash", importPolyfill: !0 }]
		)
	}
	const bundle = await rollup({
			input: filePathResolved,
			plugins: [
				rollupPluginJSON({ preferConst: !0 }),
				{
					name: "hackmud-script-manager",
					resolveId(source) {
						if (source == filePathResolved) return filePathResolved
					},
					async load(id) {
						if (id == filePathResolved) return (await preprocess(code, { uniqueId })).code
					},
					async transform(code, id) {
						if (id == filePathResolved) return
						if (isAbsolute(id) && !id.includes(`${sep}node_modules${sep}`))
							return (await preprocess(code, { uniqueId })).code
						let program
						traverse(parse(code, { sourceType: "module" }), {
							Program(path) {
								program = path
								path.skip()
							}
						})
						for (const referencePath of getReferencePathsToGlobal("JSON", program))
							"MemberExpression" == referencePath.parentPath.node.type &&
								"Identifier" == referencePath.parentPath.node.property.type &&
								("parse" == referencePath.parentPath.node.property.name
									? (referencePath.parentPath.node.property.name = "oparse")
									: "stringify" == referencePath.parentPath.node.property.name &&
										(referencePath.parentPath.node.property.name = "ostringify"))
						return generate(program.node).code
					}
				},
				babel({ babelHelpers: "bundled", plugins, configFile: !1, extensions: supportedExtensions }),
				rollupPluginCommonJS(),
				rollupPluginNodeResolve({ extensions: supportedExtensions }),
				!!rootFolderPath && rollupPluginAlias({ entries: [{ find: /^\//, replacement: rootFolderPath + "/" }] })
			],
			treeshake: { moduleSideEffects: !1 }
		}),
		seclevelNames = ["NULLSEC", "LOWSEC", "MIDSEC", "HIGHSEC", "FULLSEC"]
	code = (await bundle.generate({})).output[0].code
	const { file, seclevel, warnings } = transform(parse(code, { sourceType: "module" }), sourceCode, {
		uniqueId,
		scriptUser,
		scriptName
	})
	if (null != statedSeclevel && seclevel < statedSeclevel)
		throw Error(
			`detected seclevel ${seclevelNames[seclevel]} is lower than stated seclevel ${seclevelNames[statedSeclevel]}`
		)
	code = generate(file).code
	if (shouldMinify) code = await minify(file, { uniqueId, mangleNames, forceQuineCheats, autocomplete })
	else {
		traverse(file, {
			MemberExpression({ node: memberExpression }) {
				if (!memberExpression.computed) {
					assert("Identifier" == memberExpression.property.type, "src/processScript/index.ts:336:60")
					if ("prototype" == memberExpression.property.name) {
						memberExpression.computed = !0
						memberExpression.property = t.stringLiteral("prototype")
					} else if ("__proto__" == memberExpression.property.name) {
						memberExpression.computed = !0
						memberExpression.property = t.stringLiteral("__proto__")
					} else if (includesIllegalString(memberExpression.property.name)) {
						memberExpression.computed = !0
						memberExpression.property = t.stringLiteral(
							replaceUnsafeStrings(uniqueId, memberExpression.property.name)
						)
					}
				}
			},
			VariableDeclarator(path) {
				const renameVariables = lValue => {
					switch (lValue.type) {
						case "Identifier":
							includesIllegalString(lValue.name) &&
								path.scope.rename(
									lValue.name,
									"$" +
										Math.floor(Math.random() * 2 ** 52)
											.toString(36)
											.padStart(11, "0")
								)
							break
						case "ObjectPattern":
							for (const property of lValue.properties) {
								assert("ObjectProperty" == property.type, "src/processScript/index.ts:367:51")
								renameVariables(property.value)
							}
							break
						case "ArrayPattern":
							for (const element of lValue.elements) element && renameVariables(element)
							break
						default:
							throw Error(`unknown lValue type "${lValue.type}"`)
					}
				}
				renameVariables(path.node.id)
			},
			ObjectProperty({ node: objectProperty }) {
				if ("Identifier" == objectProperty.key.type && includesIllegalString(objectProperty.key.name)) {
					objectProperty.key = t.stringLiteral(replaceUnsafeStrings(uniqueId, objectProperty.key.name))
					objectProperty.shorthand = !1
				}
			},
			StringLiteral({ node }) {
				node.value = replaceUnsafeStrings(uniqueId, node.value)
			},
			TemplateLiteral({ node }) {
				for (const templateElement of node.quasis)
					if (templateElement.value.cooked) {
						templateElement.value.cooked = replaceUnsafeStrings(uniqueId, templateElement.value.cooked)
						templateElement.value.raw = templateElement.value.cooked
							.replaceAll("\\", "\\\\")
							.replaceAll("`", "\\`")
							.replaceAll("${", "$\\{")
					} else templateElement.value.raw = replaceUnsafeStrings(uniqueId, templateElement.value.raw)
			},
			RegExpLiteral(path) {
				path.node.pattern = replaceUnsafeStrings(uniqueId, path.node.pattern)
				delete path.node.extra
			}
		})
		code = await format(generate(file, { comments: !1 }).code, {
			parser: "babel",
			arrowParens: "avoid",
			semi: !1,
			trailingComma: "none"
		})
	}
	code = postprocess(code, uniqueId)
	if (includesIllegalString(code))
		throw Error(
			'you found a weird edge case where I wasn\'t able to replace illegal strings like "SC$", please report thx'
		)
	return { script: code, warnings }
}
export { minify, postprocess, preprocess, processScript, transform }

==> hackmud-script-manager-0.21.1-8072045.tgz: package/processScript/index.js <==
import { relative, isAbsolute, sep } from "path"
import f32fhtc0ktfq84afkvak6xwp from "@babel/generator"
import { parse } from "@babel/parser"
import jt6w4z2lvmfkblngexw9l9wg from "@babel/plugin-proposal-decorators"
import zcaqhzusgvsoi2gishyb72to from "@babel/plugin-proposal-destructuring-private"
import xes6d3s5rum7xj224qugy7on from "@babel/plugin-proposal-explicit-resource-management"
import dd0rqe9346tamfbxyvddl4yj from "@babel/plugin-transform-class-properties"
import wx29ry6muzcjh9n9g62r95gu from "@babel/plugin-transform-class-static-block"
import qcwwih7yuelcgp2zsloqnjr4 from "@babel/plugin-transform-exponentiation-operator"
import ez19e21mz73mll9wxfykyw07 from "@babel/plugin-transform-json-strings"
import vd7myomkrcmcfpm93nrx9kkc from "@babel/plugin-transform-logical-assignment-operators"
import mw53z57ogk4k0qxaulqs7hur from "@babel/plugin-transform-nullish-coalescing-operator"
import oox83tsy1y6a31sqju1hrmsx from "@babel/plugin-transform-numeric-separator"
import b8fc1ommbmr91idzubf822m5 from "@babel/plugin-transform-object-rest-spread"
import txofwbtd4i8fitzwb4zvrj3e from "@babel/plugin-transform-optional-catch-binding"
import q40h63iq1x13ej9z1p5o7nz7 from "@babel/plugin-transform-optional-chaining"
import pztmuxbkqlwf2wb0pt40zy6w from "@babel/plugin-transform-private-property-in-object"
import aic8ut1pgdh3ic5p295ge7y4 from "@babel/plugin-transform-unicode-sets-regex"
import bllgrfz6aotgd3ezg3mw89jw from "@babel/traverse"
import t from "@babel/types"
import rollupPluginAlias from "@rollup/plugin-alias"
import { babel } from "@rollup/plugin-babel"
import rollupPluginCommonJS from "@rollup/plugin-commonjs"
import rollupPluginJSON from "@rollup/plugin-json"
import rollupPluginNodeResolve from "@rollup/plugin-node-resolve"
import { assert } from "@samual/lib/assert"
import prettier from "prettier"
import { rollup } from "rollup"
import { supportedExtensions } from "../constants.js"
import { minify } from "./minify.js"
import { postprocess } from "./postprocess.js"
import { preprocess } from "./preprocess.js"
import { getReferencePathsToGlobal, includesIllegalString, replaceUnsafeStrings } from "./shared.js"
import { transform } from "./transform.js"
import "@samual/lib/countHackmudCharacters"
import "@samual/lib/spliceString"
import "acorn"
import "terser"
import "import-meta-resolve"
import "@samual/lib/clearObject"
const generate = f32fhtc0ktfq84afkvak6xwp.default,
	babelPluginProposalDecorators = jt6w4z2lvmfkblngexw9l9wg.default,
	babelPluginProposalDestructuringPrivate = zcaqhzusgvsoi2gishyb72to.default,
	babelPluginProposalExplicitResourceManagement = xes6d3s5rum7xj224qugy7on.default,
	babelPluginTransformClassProperties = dd0rqe9346tamfbxyvddl4yj.default,
	babelPluginTransformClassStaticBlock = wx29ry6muzcjh9n9g62r95gu.default,
	babelPluginTransformExponentiationOperator = qcwwih7yuelcgp2zsloqnjr4.default,
	babelPluginTransformJsonStrings = ez19e21mz73mll9wxfykyw07.default,
	babelPluginTransformLogicalAssignmentOperators = vd7myomkrcmcfpm93nrx9kkc.default,
	babelPluginTransformNullishCoalescingOperator = mw53z57ogk4k0qxaulqs7hur.default,
	babelPluginTransformNumericSeparator = oox83tsy1y6a31sqju1hrmsx.default,
	babelPluginTransformObjectRestSpread = b8fc1ommbmr91idzubf822m5.default,
	babelPluginTransformOptionalCatchBinding = txofwbtd4i8fitzwb4zvrj3e.default,
	babelPluginTransformOptionalChaining = q40h63iq1x13ej9z1p5o7nz7.default,
	babelPluginTransformPrivatePropertyInObject = pztmuxbkqlwf2wb0pt40zy6w.default,
	babelPluginTransformUnicodeSetsRegex = aic8ut1pgdh3ic5p295ge7y4.default,
	traverse = bllgrfz6aotgd3ezg3mw89jw.default,
	{ format } = prettier
async function processScript(
	code,
	{
		minify: shouldMinify = !0,
		uniqueId = Math.floor(Math.random() * 2 ** 52)
			.toString(36)
			.padStart(11, "0"),
		scriptUser,
		scriptName,
		filePath,
		mangleNames = !1,
		forceQuineCheats,
		rootFolderPath
	}
) {
	assert(/^\w{11}$/.exec(uniqueId), "src/processScript/index.ts:82:36")
	const sourceCode = code
	let autocomplete, statedSeclevel
	const autocompleteMatch = /^function\s*\(.+\/\/(?<autocomplete>.+)/.exec(code)
	if (autocompleteMatch) {
		code = "export default " + code
		;({ autocomplete } = autocompleteMatch.groups)
	} else
		for (const line of code.split("\n")) {
			const comment = /^\s*\/\/(?<commentContent>.+)/.exec(line)
			if (!comment) break
			const commentContent = comment.groups.commentContent.trim()
			if (commentContent.startsWith("@autocomplete ")) autocomplete = commentContent.slice(14).trimStart()
			else if (commentContent.startsWith("@seclevel ")) {
				const seclevelString = commentContent.slice(10).trimStart().toLowerCase()
				switch (seclevelString) {
					case "fullsec":
					case "full":
					case "fs":
					case "4s":
					case "f":
					case "4":
						statedSeclevel = 4
						break
					case "highsec":
					case "high":
					case "hs":
					case "3s":
					case "h":
					case "3":
						statedSeclevel = 3
						break
					case "midsec":
					case "mid":
					case "ms":
					case "2s":
					case "m":
					case "2":
						statedSeclevel = 2
						break
					case "lowsec":
					case "low":
					case "ls":
					case "1s":
					case "l":
					case "1":
						statedSeclevel = 1
						break
					case "nullsec":
					case "null":
					case "ns":
					case "0s":
					case "n":
					case "0":
						statedSeclevel = 0
						break
					default:
						throw Error(`unrecognised seclevel "${seclevelString}"`)
				}
			}
		}
	assert(/^\w{11}$/.exec(uniqueId), "src/processScript/index.ts:161:36")
	const plugins = [
		[babelPluginProposalDecorators, { decoratorsBeforeExport: !0 }],
		[babelPluginTransformClassProperties],
		[babelPluginTransformClassStaticBlock],
		[babelPluginTransformPrivatePropertyInObject],
		[babelPluginTransformLogicalAssignmentOperators],
		[babelPluginTransformNumericSeparator],
		[babelPluginTransformNullishCoalescingOperator],
		[babelPluginTransformOptionalChaining],
		[babelPluginTransformOptionalCatchBinding],
		[babelPluginTransformJsonStrings],
		[babelPluginTransformObjectRestSpread],
		[babelPluginTransformExponentiationOperator],
		[babelPluginTransformUnicodeSetsRegex],
		[babelPluginProposalDestructuringPrivate],
		[babelPluginProposalExplicitResourceManagement]
	]
	let filePathResolved
	if (filePath) {
		filePathResolved = relative(".", filePath)
		if (filePath.endsWith(".ts"))
			plugins.push([
				await import("@babel/plugin-transform-typescript").then(module => module.default),
				{ allowDeclareFields: !0, optimizeConstEnums: !0 }
			])
		else {
			const [
				babelPluginProposalDoExpressions,
				babelPluginProposalFunctionBind,
				babelPluginProposalFunctionSent,
				babelPluginProposalPartialApplication,
				babelPluginProposalPipelineOperator,
				babelPluginProposalThrowExpressions,
				babelPluginProposalRecordAndTuple
			] = await Promise.all([
				import("@babel/plugin-proposal-do-expressions").then(module => module.default),
				import("@babel/plugin-proposal-function-bind").then(module => module.default),
				import("@babel/plugin-proposal-function-sent").then(module => module.default),
				import("@babel/plugin-proposal-partial-application").then(module => module.default),
				import("@babel/plugin-proposal-pipeline-operator").then(module => module.default),
				import("@babel/plugin-proposal-throw-expressions").then(module => module.default),
				import("@babel/plugin-proposal-record-and-tuple").then(module => module.default)
			])
			plugins.push(
				[babelPluginProposalDoExpressions],
				[babelPluginProposalFunctionBind],
				[babelPluginProposalFunctionSent],
				[babelPluginProposalPartialApplication],
				[babelPluginProposalPipelineOperator, { proposal: "hack", topicToken: "%" }],
				[babelPluginProposalThrowExpressions],
				[babelPluginProposalRecordAndTuple, { syntaxType: "hash", importPolyfill: !0 }]
			)
		}
	} else {
		filePathResolved = uniqueId + ".ts"
		const [
			babelPluginTransformTypescript,
			babelPluginProposalDoExpressions,
			babelPluginProposalFunctionBind,
			babelPluginProposalFunctionSent,
			babelPluginProposalPartialApplication,
			babelPluginProposalPipelineOperator,
			babelPluginProposalThrowExpressions,
			babelPluginProposalRecordAndTuple
		] = await Promise.all([
			import("@babel/plugin-transform-typescript").then(module => module.default),
			import("@babel/plugin-proposal-do-expressions").then(module => module.default),
			import("@babel/plugin-proposal-function-bind").then(module => module.default),
			import("@babel/plugin-proposal-function-sent").then(module => module.default),
			import("@babel/plugin-proposal-partial-application").then(module => module.default),
			import("@babel/plugin-proposal-pipeline-operator").then(module => module.default),
			import("@babel/plugin-proposal-throw-expressions").then(module => module.default),
			import("@babel/plugin-proposal-record-and-tuple").then(module => module.default)
		])
		plugins.push(
			[babelPluginTransformTypescript, { allowDeclareFields: !0, optimizeConstEnums: !0 }],
			[babelPluginProposalDoExpressions],
			[babelPluginProposalFunctionBind],
			[babelPluginProposalFunctionSent],
			[babelPluginProposalPartialApplication],
			[babelPluginProposalPipelineOperator, { proposal: "hack", topicToken: "%" }],
			[babelPluginProposalThrowExpressions],
			[babelPluginProposalRecordAndTuple, { syntaxType: "hash", importPolyfill: !0 }]
		)
	}
	const bundle = await rollup({
			input: filePathResolved,
			plugins: [
				rollupPluginJSON({ preferConst: !0 }),
				{
					name: "hackmud-script-manager",
					resolveId(source) {
						if (source == filePathResolved) return filePathResolved
					},
					async load(id) {
						if (id == filePathResolved) return (await preprocess(code, { uniqueId })).code
					},
					async transform(code, id) {
						if (id == filePathResolved) return
						if (isAbsolute(id) && !id.includes(`${sep}node_modules${sep}`))
							return (await preprocess(code, { uniqueId })).code
						let program
						traverse(parse(code, { sourceType: "module" }), {
							Program(path) {
								program = path
								path.skip()
							}
						})
						for (const referencePath of getReferencePathsToGlobal("JSON", program))
							"MemberExpression" == referencePath.parentPath.node.type &&
								"Identifier" == referencePath.parentPath.node.property.type &&
								("parse" == referencePath.parentPath.node.property.name
									? (referencePath.parentPath.node.property.name = "oparse")
									: "stringify" == referencePath.parentPath.node.property.name &&
										(referencePath.parentPath.node.property.name = "ostringify"))
						return generate(program.node).code
					}
				},
				babel({ babelHelpers: "bundled", plugins, configFile: !1, extensions: supportedExtensions }),
				rollupPluginCommonJS(),
				rollupPluginNodeResolve({ extensions: supportedExtensions }),
				!!rootFolderPath && rollupPluginAlias({ entries: [{ find: /^\//, replacement: rootFolderPath + "/" }] })
			],
			treeshake: { moduleSideEffects: !1 }
		}),
		seclevelNames = ["NULLSEC", "LOWSEC", "MIDSEC", "HIGHSEC", "FULLSEC"]
	code = (await bundle.generate({})).output[0].code
	const { file, seclevel, warnings } = transform(parse(code, { sourceType: "module" }), sourceCode, {
		uniqueId,
		scriptUser,
		scriptName
	})
	if (null != statedSeclevel && seclevel < statedSeclevel)
		throw Error(
			`detected seclevel ${seclevelNames[seclevel]} is lower than stated seclevel ${seclevelNames[statedSeclevel]}`
		)
	code = generate(file).code
	if (shouldMinify) code = await minify(file, { uniqueId, mangleNames, forceQuineCheats, autocomplete })
	else {
		traverse(file, {
			MemberExpression({ node: memberExpression }) {
				if (!memberExpression.computed) {
					assert("Identifier" == memberExpression.property.type, "src/processScript/index.ts:336:60")
					if ("prototype" == memberExpression.property.name) {
						memberExpression.computed = !0
						memberExpression.property = t.stringLiteral("prototype")
					} else if ("__proto__" == memberExpression.property.name) {
						memberExpression.computed = !0
						memberExpression.property = t.stringLiteral("__proto__")
					} else if (includesIllegalString(memberExpression.property.name)) {
						memberExpression.computed = !0
						memberExpression.property = t.stringLiteral(
							replaceUnsafeStrings(uniqueId, memberExpression.property.name)
						)
					}
				}
			},
			VariableDeclarator(path) {
				const renameVariables = lValue => {
					switch (lValue.type) {
						case "Identifier":
							includesIllegalString(lValue.name) &&
								path.scope.rename(
									lValue.name,
									"$" +
										Math.floor(Math.random() * 2 ** 52)
											.toString(36)
											.padStart(11, "0")
								)
							break
						case "ObjectPattern":
							for (const property of lValue.properties) {
								assert("ObjectProperty" == property.type, "src/processScript/index.ts:367:51")
								renameVariables(property.value)
							}
							break
						case "ArrayPattern":
							for (const element of lValue.elements) element && renameVariables(element)
							break
						default:
							throw Error(`unknown lValue type "${lValue.type}"`)
					}
				}
				renameVariables(path.node.id)
			},
			ObjectProperty({ node: objectProperty }) {
				if ("Identifier" == objectProperty.key.type && includesIllegalString(objectProperty.key.name)) {
					objectProperty.key = t.stringLiteral(replaceUnsafeStrings(uniqueId, objectProperty.key.name))
					objectProperty.shorthand = !1
				}
			},
			StringLiteral({ node }) {
				node.value = replaceUnsafeStrings(uniqueId, node.value)
			},
			TemplateLiteral({ node }) {
				for (const templateElement of node.quasis)
					if (templateElement.value.cooked) {
						templateElement.value.cooked = replaceUnsafeStrings(uniqueId, templateElement.value.cooked)
						templateElement.value.raw = templateElement.value.cooked
							.replaceAll("\\", "\\\\")
							.replaceAll("`", "\\`")
							.replaceAll("${", "$\\{")
					} else templateElement.value.raw = replaceUnsafeStrings(uniqueId, templateElement.value.raw)
			},
			RegExpLiteral(path) {
				path.node.pattern = replaceUnsafeStrings(uniqueId, path.node.pattern)
				delete path.node.extra
			}
		})
		code = await format(generate(file, { comments: !1 }).code, {
			parser: "babel",
			arrowParens: "avoid",
			semi: !1,
			trailingComma: "none"
		})
	}
	code = postprocess(code, uniqueId)
	if (includesIllegalString(code))
		throw Error(
			'you found a weird edge case where I wasn\'t able to replace illegal strings like "SC$", please report thx'
		)
	return { script: code, warnings }
}
export { minify, postprocess, preprocess, processScript, transform }
